/* QWalk (qwmeasure.c) 
 * Copyright (C) 2007  Franklin Marquezino
 *                                                                                                                           
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *                                                                                                                           
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA
 */

#include<stdio.h>
#include<stdlib.h>
#include<complex.h>
#include<math.h>
#include "qwmem_complex.h"
#include "qwmem_real.h"
#include "qwmeasure.h"
#include "qwconsts.h" 


int measureState2D(double complex *****A, double complex *****Atemp, 
		   int max, int detectors, int **detector_pts){
  int j, k, m, n;
  int det, dice, result, error;
  double *p, *sp;
  double complex ****aux;

  if( (!*A) || (!*Atemp))
    return -1;
  if(max<1)
    return -2;
  if(detectors<0)
    return -3;
  if(!detector_pts)
    return -4;

  p = allocReal1D(detectors+1);
  if(!p)
    return -5;
  error = cleanReal1D(p, detectors+1);
  if(error)
    return -6;

  sp = allocReal1D(detectors+1);
  if(!sp)
    return -5;
  error = cleanReal1D(sp, detectors+1);
  if(error)
    return -6;

  /* Here we evaluate the probability of each detection... */
  for(det=1; det<=detectors; det++ ){  
    m = max + detector_pts[det][0];
    n = max + detector_pts[det][1];

    for(j=0; j<2; j++)
      for(k=0; k<2; k++)
	p[det] += (*A)[j][k][m][n]*conj((*A)[j][k][m][n]);
  }

  /* ...and the probability of measuring the complement */
  p[0] = 1.0;
  for(j=1; j<=detectors; j++)
    p[0] -= p[j];

  /* ...  */
  sp[0] = p[0];
  for(det=1; det<=detectors; det++)
    sp[det] = sp[det-1] + p[det];

  /* We get a new random number */
  dice = rand();

  /* Based on this number we identify the corresponding result */
  for(result=0; result<=detectors; result++)
    if(dice <= sp[result]*RAND_MAX) break;

  for(m=-max; m<=max; m++){
    for(n=-max; n<=max; n++){
      int delta;

      if(result==0){
	delta = 1;
	for(det=1; det<=detectors; det++)
	  delta *= (1-DELTA(m, detector_pts[det][0])*
		    DELTA(n, detector_pts[det][1]));
      }
      else{
	delta = DELTA(m, detector_pts[result][0])*
	  DELTA(n, detector_pts[result][1]);
      }
      
      for(j=0; j<2; j++)
	for(k=0; k<2; k++)
	  (*Atemp)[j][k][max+m][max+n] = 
	    delta*((*A)[j][k][max+m][max+n])/sqrt(p[result]);
    }
  }

  free(p);
  free(sp);

  /* Now we quicky exchange matrices A and Atemp */
  aux = *A;
  *A = *Atemp;
  *Atemp = aux;

  error = cleanComplex4D(*Atemp, 2, 2, 2*max+1, 2*max+1);
  if(error)
    return -6;

  return result;
}
