/* QWalk (qwstate_io.c) 
 * Copyright (C) 2007  Franklin Marquezino
 *                                                                                                                           
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *                                                                                                                           
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA
 */

#include<stdio.h>
#include<stdlib.h>
#include<complex.h>
#include<string.h>
#include<math.h>
#include<limits.h>
#include<time.h>
#include "qwmem_complex.h"
#include "qwstate_io.h"
#include "qwconsts.h"


double complex **readStateFile1D(const char *filename, int max, unsigned int lattType){
  FILE *in;
  double complex **state;
  int j,m;
  double real, imag;
  char keyword[100];

  in = fopen(filename,"rt");
  if(!in)
    return NULL;

  /* First we search the BEGINSTATE keyword,...*/ 
  do{
    fscanf(in,"%s",keyword);
  }while(STRNEQ(keyword,"BEGINSTATE") && !feof(in));

  if(feof(in))
    return NULL;

  state = (lattType == LINE_LATT) ?
    allocComplex2D(2, 2*max+1) : allocComplex2D(2, max);
  if(!state)
    return NULL;

  if(lattType == LINE_LATT)
    cleanComplex2D(state, 2, 2*max+1);
  else
    cleanComplex2D(state, 2, max);

  /* ...when we find it we start reading integer numbers, describing 
   * the "kets", and complex numbers, describing the amplitudes. 
   * Then, we assign each complex amplitude to the corresponding 
   * entry of the complex matrix. The complex numbers must be 
   * given in the input file with the real and imaginary parts 
   * separated by a space. The amplitudes of those "kets" not 
   * described in the input file are all zero by default.
   */  
  fscanf(in,"%d", &j);
  fscanf(in,"%d", &m);
  fscanf(in,"%lf", &real);
  fscanf(in,"%lf", &imag);
  while(j!=-1){
    if(lattType == LINE_LATT)
      state[j][max+m] = real+ I*imag;
    else
      state[j][m] = real+ I*imag;
    fscanf(in,"%d", &j);
    fscanf(in,"%d", &m);
    fscanf(in,"%lf", &real);
    fscanf(in,"%lf", &imag);
  }
  
  fclose(in);
  return state;
}



double complex ****readStateFile2D(const char *filename, int max){
  FILE *in;
  double complex ****state;
  int j,k,m,n;
  double real, imag;
  char keyword[100];

  in = fopen(filename,"rt");
  if(!in)
    return NULL;

  /* First we search the BEGINSTATE keyword,...*/
  do{
    fscanf(in,"%s",keyword);
  }while(STRNEQ(keyword,"BEGINSTATE") && !feof(in));

  if(feof(in))
    return NULL;

  state = allocComplex4D(2, 2, 2*max+1, 2*max+1);
  if(!state)
    return NULL;

  cleanComplex4D(state, 2, 2, 2*max+1, 2*max+1);


  /* ...when we find it we start reading integer numbers, describing 
   * the "kets", and complex numbers, describing the amplitudes. 
   * Then, we assign each complex amplitude to the corresponding 
   * entry of the complex matrix. The complex numbers must be 
   * given in the input file with the real and imaginary parts 
   * separated by a space. The amplitudes of those "kets" not 
   * described in the input file are all zero by default.
   */  
  fscanf(in,"%d", &j);
  fscanf(in,"%d", &k);
  fscanf(in,"%d", &m);
  fscanf(in,"%d", &n);
  fscanf(in,"%lf", &real);
  fscanf(in,"%lf", &imag);
  while(j!=-1 && k!=-1){
    state[j][k][max+m][max+n] = real+ I*imag;
    fscanf(in,"%d", &j);
    fscanf(in,"%d", &k);
    fscanf(in,"%d", &m);
    fscanf(in,"%d", &n);
    fscanf(in,"%lf", &real);
    fscanf(in,"%lf", &imag);
  }

  fclose(in);
  return state;
}
